// APPN07_Horloge
/* Auteur: Brock Craft 
 *
 * Horloge
 * Librairies Wire, LiquidCrystal et Adafruit RTC
 *
 * v0.1 30.04.2013 VF par OE 1606
 * Adapté de http://www.adafruit.com/products/746
*/

#include <Wire.h>          // Librairie I2C Wire pour dialogue avec le DS1307
#include "RTClib.h"        // Fonctions de date et heure pour le DS1307
#include <LiquidCrystal.h> // Fonctions d'affichage pour le LCD

RTC_DS1307 rtc;                         // Objet horloge rtc
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);  // Objet afficheur lcd 

DateTime present;        // Objet DateTime pour lire le module RTC

boolean affichageAlarme = false;  // Affichage de l'heure ou de l'alarme
boolean alarmeEnCours = false;    // Alarme réglée ?
boolean alarmeArmee = false;      // Alarme armée ?

int heureAL = 21;   // Choix d'une heure d'alarme initiale
int minuteAL = 59;  // OE: Ne pas commencer par un chiffre 0

// Broches de gestion d'alarme
const int broALReglage = 6;           // Pour basculer en réglage d'alarme
const int broALAugH  = 7; // Pour incrémenter l'heure d'alarme
const int broALAugM = 8; // Pour incrémenter la minute d'alarme
const int broBuzzer = 9;              // Pour le buzzer
const int broALActive = 10;        // Pour armer l'alarme


void setup() {
  Wire.begin();      // Active les communications pour le LCD
  rtc.begin();       // Active le RTC
  lcd.begin(16, 2);  // Active le LCD
  lcd.print(" Editions FIRST ");  // Affichage de message
  delay(1000);                    // Petite pause pour lire
  lcd.clear();       // Vide l'écran LCD

  // Configuration de broches
  pinMode(broALReglage, INPUT);   
  pinMode(broALAugH, INPUT);
  pinMode(broALAugM, INPUT);
  pinMode(broBuzzer, OUTPUT);
  pinMode(broALActive, INPUT);
}

void loop () {
  present = rtc.now(); // Lit l'heure
 
  int alarmeActivee = digitalRead(broALActive);
  if (alarmeActivee == HIGH){
   alarmeArmee=true;
  } else {
   alarmeArmee=false;
  }

  // Sonner le buzzer ou pas
 if(alarmeArmee) {        // SI l'alarme est ON mais
  if (!alarmeEnCours) {   //  SI elle ne sonne pas encore,
    testerAlarme();       //   voir si elle doit sonner
  } 
  else {
    sonnerAlarme();      // Sinon, on la fait sonner
  }
}
  // Si en mode Réglage de l'alarme
  int reglageAL = digitalRead(broALReglage);  // Lire broche réglage
  if (reglageAL==HIGH){      // Si mode choisi
    affichageAlarme=true;  // Drapeau utilisé par reafficher()
    reglerAlarme();        // Réglage de l'heure d'alarme
  }
  else {                   // Si pas en réglage d'alarme
    affichageAlarme=false; // configurer le drapeua
  }

  reafficher();
}

// Réaffiche soit l'heure courante, soit l'heure d'alarme
void reafficher(){
  if(affichageAlarme) {   // Si en mode réglage de l'alarme
    lcd.setCursor(0, 0);  // Pose le curseur en col 0 ligne 0
    lcd.print("Heure ALARME : ");
    lcd.setCursor(4, 1);  // Envoie le curseur en col 4 du bas
    lcd.print(" ");       // Efface les chiffres de l'heure 
    lcd.setCursor(5, 1);  // Avance d'une position pour centrer
    if (heureAL<10){     // Si entre 0 et 9 , il manque la dizaine
      lcd.print(0);       // ... et on ajoute donc un 0.
    }  
    lcd.print(heureAL);  // Affiche l'heure d'alarme
    lcd.setCursor(7, 1);  // Colonne suivante
    lcd.print(":");       // Affiche le séparateur
    lcd.setCursor(8, 1);  // Colonne suivante
    if (minuteAL<10){    // AJout du 0 comme pour l'heure
      lcd.print(0);       // ... si moins de 10
    }  
    lcd.print(minuteAL); // Affiche les minutes de l'alarme
    lcd.setCursor(10, 1); // Colonne suivante
    lcd.print("  ");      // Efface les chiffres
  } 
  else {                  // Si pas en mode réglage d'alarme, afficher l'heure
    int h = present.hour();   // Extrait les chiffres de l'heure et les place dans h
    int m = present.minute(); // Idem pour les minutes dans l'entier m
    int s = present.second(); // Idem pour les secondes dans l'entier s

    lcd.setCursor(0, 0);  // Pose le curseur en col 0 ligne 0
    if(alarmeArmee){
      lcd.print("*Horloge Arduino");
    }
    else {
      lcd.print(" Horloge Arduino"); 
    }
    lcd.setCursor(4, 1);  // Envoie le curseur en col 4 du bas
    if (h<10){            // Ajout de la dizaine si nécessaire
      lcd.print(0); 
    }  
    lcd.print(h);         // Affiche l'heure actuelle
    lcd.setCursor(6, 1);  // Colonne suivante
    lcd.print(":");
    lcd.setCursor(7, 1);
    if (m<10){
      lcd.print(0); 
    }  
    lcd.print(m);         // Affiche les minutes actuelles
    lcd.setCursor(9, 1);  // Colonne suivante
    lcd.print(":");     
    lcd.setCursor(10, 1);
    if (s<10){
      lcd.print(0);
    }  
    lcd.print(s);         // Affiche les secondes actuelles
  }
}

// Teste si l'alarme doit sonner
void testerAlarme(){
  if(heureAL==present.hour() && minuteAL==present.minute() && present.second()==0){ 
    alarmeEnCours=true;  // Autorise le déclenchement au prochain tour
  } 
}

// Incrémente heure et minute
void reglerAlarme(){
  int boutonALH=digitalRead(broALAugH); 
  int boutonALM=digitalRead(broALAugM);

  if (boutonALH==HIGH){ // Si poussoir des heures enfoncé
    heureAL+=1;        // Fait avancer les heures de 1
    delay(200);         // Pause avant de continuer
    if(heureAL>23){    // Si l'heure dépasse 23, la ramener à 0
      heureAL=0; 
    }
  }
  if (boutonALM==HIGH){  // Si poussoir des minutes enfoncé
    minuteAL+=1; 
    delay(200);
    if(minuteAL>59){    // Si la minute dépasse 59, la ramener à 0
      minuteAL=0; 
    }
  }
}

void sonnerAlarme() {
  float frequenceAL=800;           // Fréquence du signal d'alarme
  float periodeAL = (1.0 / frequenceAL) * 1000000;
  long  dureeBip = 250000;         // Durée en microsecondes
  long  tempsPasse = 0;

  while (tempsPasse < dureeBip) {
    digitalWrite(broBuzzer,HIGH);
    delayMicroseconds(periodeAL / 2);
    digitalWrite(broBuzzer, LOW);
    delayMicroseconds(periodeAL / 2);
    tempsPasse += (periodeAL);
  }
  delayMicroseconds(dureeBip);

  // Stoppe l'alarme audio si un bouton est enfoncé
  int boutonALH=digitalRead(broALAugH); 
  int boutonALM=digitalRead(broALAugM);

  if (boutonALH==HIGH || boutonALM==HIGH){
    alarmeEnCours=false;
  }
}


