// APPN09_RFID
/* Auteur: Brock Craft 
 *
 * Lecteur de badges RFID
 *
 * v0.1 30.04.2013
 * Adapté de bildr.com
 * VF par OE 160625
*/

const int broRelais = 11;     // Broche du relais
const int broLED = 12;        // Broche de la LED
const int broRFIDReset = 13;  // Broche de Reset
const int tpsRelaisON = 5000; // Temps de travail du relais

// Liste d'identifiants RFID connus.
// Au départ, laissez-en un, même faux.
// Sinon, la comparaison plantera.
// Puis indiquez ceux lus sur le moniteur série.

char tag1[13] = "070050859C4E";  // ID de votre badge de test ici
char tag2[13] = "010203AABBCC";  // Exemple
char tag3[13] = "010203AABBDD";  // Exemple
// etc. si autres tags

void setup(){
  Serial.begin(9600);               // Ouvre la connexion série en TX/RX
  pinMode(broRFIDReset, OUTPUT);    // Redémarre le lecteur
  pinMode(broLED, OUTPUT);          // Broche LED en sortie
  pinMode(broRelais, OUTPUT);       // Broche relais en sortie
  digitalWrite(broRFIDReset, HIGH); // Prêt à lire
  Serial.println("PRET.");          // Confirme dans la console
}

void loop(){
  Serial.println("Recherche de tag...");
  char chaineTag[13];      // Tableau de stockage du tag lu
  int index = 0;           // Indice dans le tableau
  boolean lecEnCours = false; // Drapeau à 1 si lecture

  while(Serial.available()) {  // Si Moniteur série
    int ocLu = Serial.read();      // Lit le prochain octet
    if(ocLu == 2) lecEnCours = true;  // 2 = début de tag
    if(ocLu == 3) lecEnCours = false; // 3 = fin de tag
    
    // Si lecEnCours et ni début, ni fin, stockage
    if(lecEnCours && ocLu != 2 && ocLu != 10 && ocLu != 13){
      chaineTag[index] = ocLu;  // Position de l'indice
      index ++;                 // On avance d'un élément
    }
  }
  testerTag(chaineTag);  // Tag reconnu ?
  effacerTag(chaineTag); // Vide le tableau
  reinitLecteur();       // Reset du lecteur
}

void testerTag(char tag[]){
  // Compare la lecture à chaque tag attendu
  if(strlen(tag) == 0) return; // Si vide, on s'en va

  if(comparerTag(tag, tag1)){   // Si c'est tag1:
    allumerLED(); 
    fermerRelais();
  }
  else if(comparerTag(tag, tag2)){ // Si tag2:
    allumerLED();
  }
  else if(comparerTag(tag, tag3)){ // Si tag3:
    allumerLED();
  } else {                        // Si aucun des trois
    Serial.println("Un nouveau tag : "); 
    Serial.println(tag);          // Affiche le code à récupérer
    delay(5000);                  // Pause de 5 sec. pour noter
  }
}

boolean comparerTag(char tablo1[], char tablo2[]){
  // Compare les deux chaînes en cherchant
  // si la lecture est un tag connu
  if(strlen(tablo1) == 0) return false; // Si vide, on s'en va
  for(int i = 0; i < 12; i++){
    if(tablo1[i] != tablo2[i]) return false; // Si pas idem, on renvoie faux
  }
  Serial.println("Tag valide !");
  return true; // Sinon, on renvoie vrai
}

void allumerLED(){
  digitalWrite(broLED, HIGH);  // Allumer la LED
  delay(250);                  // Attendre
  digitalWrite(broLED, LOW);   // Eteindre la LED
}

void fermerRelais(){
  digitalWrite(broRelais, HIGH);  // Activer le relais
  delay(tpsRelaisON);             // Attendre
  digitalWrite(broRelais, LOW);   // Couper le relais
}

void effacerTag(char tablo1[]){
  // Vide le tableau lecEnCours avec des ASCII 0
  // Si pas mis à 0, pourrait indiquer une lecture
  for(int i = 0; i < strlen(tablo1); i++){
    tablo1[i] = 0; 
  }
} 
  
void reinitLecteur(){
  // Replace le lecteur en condition de lecture
  digitalWrite(broRFIDReset, LOW);
  digitalWrite(broRFIDReset, HIGH);
  delay(150);
}


