/* Arduino Projects for Dummies
 * by Brock Craft 
 *
 * Chapter 14: Building a Remote Controlled Car
 * Detects signals from a garden variety remote control 
 * to drive two servo motors. 
 * 
 * Uses the standard Arduino servo library and
 * the IR Remote Library by Ken Shirriff:
 * https://github.com/shirriff/Arduino-IRremote
 * Adapted from code by Michael Margolis
 *
 * v0.1 30.04.2013
*/

#include <Servo.h>
#include <IRremote.h>

const int broCapteurIR = 2; 
IRrecv oRecInfra(broCapteurIR);
decode_results oResultat;

const int broMotoD = 9;
const int broMotoG = 10;

Servo servoD;
Servo servoG;

int rotaD = 90;
int rotaG = 90;

long teleCode = 0;

boolean MIZOPOINT = false; // Valeur true pour la mise au point

void setup()
{
  if(MIZOPOINT){
    Serial.begin(9600);
  }
  oRecInfra.enableIRIn();
  servoG.attach(9);
  servoD.attach(10);
  pinMode(broMotoD, OUTPUT);
  pinMode(broMotoG, OUTPUT);
}

void loop() {
  if( oRecInfra.decode(&oResultat) )
  {
    teleCode = oResultat.value;
    if(teleCode != -1)
    {
      switch (teleCode){
        case 50174055:  // Indiquez pour chaque case votre valeur !
        Serial.println("Avance");
        rotaG -= 1;  // Valeurs inverses
        rotaD += 1; 
        break;
        
        case 50182215:  // A remplacer
        Serial.println("Recule");
        rotaG += 1;  // Inverse de l'avance
        rotaD -= 1; 
        break; 
        
        case 50168955:  // A remplacer
        Serial.println("Stop");
        rotaG = 90;  // 90 pour s'immobiliser
        rotaD = 90; 
        break; 

        case 50152125:  // A remplacer
        Serial.println("Tourne à gauche");
        rotaG -= 1;
        rotaD -= 1;  
        break; 
        
        case 50135805:  // A remplacer
        Serial.println("Tourne à droite");  // Wheels move in opposite directions
        rotaG += 1;
        rotaD += 1;
        break; 
        
        case 50139885:  // A remplacer
        Serial.println("TURBO!!");  // need to move left servo to go right
        rotaG = rotaG-50;
        rotaD = rotaD+50;
        break; 
      }
      if(MIZOPOINT){
        Serial.println(teleCode);
        afficherData();
        Serial.print(rotaG);
        Serial.print(", ");
        Serial.println(rotaD);
      }
    }
    oRecInfra.resume(); // Valeur suivante
  }
  piloterMoteurs();
  delay(10);
}

void afficherData()
{
  if (oResultat.decode_type == UNKNOWN) {
    Serial.println("-Message inconnu");
  }
  else {
    if (oResultat.decode_type == NEC) {
      Serial.print("- decode NEC: ");
    }
    else if (oResultat.decode_type == SONY) {
      Serial.print("- decode SONY: ");
    }
    else if (oResultat.decode_type == RC5) {
      Serial.print("- decode RC5: ");
    }
    else if (oResultat.decode_type == RC6) {
      Serial.print("- decode RC6: ");
    }
    Serial.print("Valeur = ");
    Serial.println( oResultat.value, DEC);
  }
}

void piloterMoteurs(){
  servoG.write(rotaG);
  servoD.write(rotaD);
}
