// APPN15_CubeInclude.ino
// Version bis avec tableau de motifs externalisé !
// par O. Engler v0.1 d20160702
//

#include "APPN15_CubeInclude.h"

const int dimCube = 3;      // Dimensions d'un côté de cube
const int ledPCou = 9;      // LED par couche
const int tpsAffCou = 1000; // Temps d'aff. une couche (microsec.)
const int vitesseMAJ = 100; // Facteur pour tempsMot en millisec.

byte tMotif[ledPCou];  // Motif courant lu dans tabMotifs
int  xMotif;           // Indice de ligne dans motif courant
int  xTabMot;          // Position pendant affichage
byte tempsMot;         // Durée d'étape/motif avant x100
unsigned long tempsFin;   // Fin d'étape (frame)

int ligLed; // Compteur de lignes de LED en réactu
int colLed; // Compteur de colonnes de LED en réactu
int xLed;   // Compteur total de LED en réactu

int broAno[] = {2,3,4,5,6,7,8,9,10};
int broCat[] = {14,15,16};  // Broches A0,A1,A2

void setup()
{
  Serial.begin(9600); // OE
  for (int i=0; i<ledPCou; i++) {
    pinMode(broAno[i], OUTPUT);
  }
  for (int i=0; i<dimCube; i++) {
    pinMode(broCat[i], OUTPUT);
  }
}

void loop()
{
  // Pointe xMotif sur le début du tableau
  xMotif = 0;
  // Lit le tableau tant que tempsMot > 0
  do {
    memcpy_P( tMotif, tabMotifs+xMotif, ledPCou );
    // Avance d'une couche
    xMotif += ledPCou;
    // Lit tempsMot depuis la mémoire PROGMEM et avance l'index.
    // tempsMot est le 10eme élément.
    tempsMot = pgm_read_byte_near( tabMotifs + xMotif++ );
    // Calcule tempsFin avec horloge en ms et tempsMot 
    tempsFin = millis() + tempsMot * vitesseMAJ;
    
    // Tant que tempsMot>0 et horloge < tempsFin
    while ( millis() < tempsFin ) {
      xTabMot = 0; // Retour en début de tableau
      // Couche par couche
      for (int couche = 0; couche < dimCube; couche++) {
        // Eteint la couche préc.
        if (couche == 0) {
          digitalWrite(broCat[dimCube-1], HIGH);
        } 
        else {
          digitalWrite(broCat[couche-1], HIGH);
        }
        xLed = 0;
        for (ligLed=0; ligLed<dimCube; ligLed++) {
          for (colLed=0; colLed<dimCube; colLed++) {
            digitalWrite( broAno[xLed++], 
             tMotif[xTabMot] & (1 << colLed) );
            Serial.print(broAno[xLed++]); // OE
            Serial.print(tMotif[xTabMot] & (1 << colLed)); // OE
          }
          xTabMot++;
          Serial.println(": ");  // OE
          Serial.print("for2 : ");  // OE
          Serial.println(xTabMot);  // OE
        }
        // Affiche la couche
        digitalWrite(broCat[couche], LOW);
        Serial.println(broCat[couche]);  // OE
        // Pause de tpsAffCou
        delayMicroseconds(tpsAffCou * 1000);  //OE
      } // Fin de for
    } // Fin de while
  } 
  while (tempsMot > 0); // 0 marque la fin du tableau de motifs
}

